@@ -5,12 +5,12 @@ Documentation for soxspipe can be found here: http://soxspipe.readthedocs.org
 
 Usage:
     soxspipe init
-    soxspipe mbias <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>] 
-    soxspipe mdark <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
-    soxspipe mflat <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
-    soxspipe disp_sol <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
-    soxspipe order_centres <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
-    soxspipe spat_sol <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
+    soxspipe [-V] mbias <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>] 
+    soxspipe [-V] mdark <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
+    soxspipe [-V] mflat <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
+    soxspipe [-V] disp_sol <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
+    soxspipe [-V] order_centres <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
+    soxspipe [-V] spat_sol <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
 
 Options:
     init                                   setup the soxspipe settings file for the first time
@@ -26,6 +26,7 @@ Options:
     -h, --help                             show this help message
     -v, --version                          show version
     -s, --settings <pathToSettingsFile>    the settings file
+    -V, --verbose                          more verbose output
 """
 ################# GLOBAL IMPORTS ####################
 import sys
@@ -118,6 +119,8 @@ def main(arguments=None):
             pickleMe[k] = theseLocals[k]
         pickle.dump(pickleMe, open(pathToPickleFile, "wb"))
 
+    verbose = a['verboseFlag']
+
     # PACK UP SOME OF THE CL SWITCHES INTO SETTINGS DICTIONARY
     if a['outputDirectory']:
         settings["intermediate-data-root"] = a['outputDirectory']
@@ -143,7 +146,8 @@ def main(arguments=None):
         recipe = soxs_mbias(
             log=log,
             settings=settings,
-            inputFrames=a["inputFrames"]
+            inputFrames=a["inputFrames"],
+            verbose=verbose
         )
         mbiasFrame = recipe.produce_product()
         print("You can find the master bias frame at `%(mbiasFrame)s`" % locals())
@@ -153,7 +157,8 @@ def main(arguments=None):
         recipe = soxs_mdark(
             log=log,
             settings=settings,
-            inputFrames=a["inputFrames"]
+            inputFrames=a["inputFrames"],
+            verbose=verbose
         )
         mdarkFrame = recipe.produce_product()
         print("You can find the master bias frame at `%(mdarkFrame)s`" % locals())
@@ -163,7 +168,8 @@ def main(arguments=None):
         disp_map = soxs_disp_solution(
             log=log,
             settings=settings,
-            inputFrames=a["inputFrames"]
+            inputFrames=a["inputFrames"],
+            verbose=verbose
         ).produce_product()
         print(f"\nSingle pinhole first guess dispersion map saved to: {disp_map}")
 
@@ -172,7 +178,8 @@ def main(arguments=None):
         order_table = soxs_order_centres(
             log=log,
             settings=settings,
-            inputFrames=a["inputFrames"]
+            inputFrames=a["inputFrames"],
+            verbose=verbose
         ).produce_product()
         print(f"\nThe order centre locations have been saved to an order table: {order_table}")
 
@@ -181,7 +188,8 @@ def main(arguments=None):
         disp_map = soxs_spatial_solution(
             log=log,
             settings=settings,
-            inputFrames=a["inputFrames"]
+            inputFrames=a["inputFrames"],
+            verbose=verbose
         ).produce_product()
         print(f"\nFull 2D dispersion map saved to: {disp_map}")
 
@@ -190,10 +198,10 @@ def main(arguments=None):
         recipe = soxs_mflat(
             log=log,
             settings=settings,
-            inputFrames=a["inputFrames"]
+            inputFrames=a["inputFrames"],
+            verbose=verbose
         )
         mflatFrame = recipe.produce_product()
-        print(f"\nThe master flat frame has been saved to: {mflatFrame}")
 
     # CALL FUNCTIONS/OBJECTS
 
