@@ -47,6 +47,8 @@ class create_dispersion_map(object):
         - ``settings`` -- the settings dictionary
         - ``pinholeFrame`` -- the calibrated pinhole frame (single or multi)
         - ``firstGuessMap`` -- the first guess dispersion map from the `soxs_disp_solution` recipe (needed in `soxs_spat_solution` recipe). Default *False*.
+        - ``qcTable`` -- the data frame to collect measured QC metrics
+        - ``productsTable`` -- the data frame to collect output products
 
     **Usage:**
 
@@ -66,13 +68,17 @@ class create_dispersion_map(object):
             log,
             settings,
             pinholeFrame,
-            firstGuessMap=False
+            firstGuessMap=False,
+            qcTable=False,
+            productsTable=False
     ):
         self.log = log
         log.debug("instantiating a new 'create_dispersion_map' object")
         self.settings = settings
         self.pinholeFrame = pinholeFrame
         self.firstGuessMap = firstGuessMap
+        self.qc = qcTable
+        self.products = productsTable
 
         # KEYWORD LOOKUP OBJECT - LOOKUP KEYWORD FROM DICTIONARY IN RESOURCES
         # FOLDER
@@ -82,6 +88,7 @@ class create_dispersion_map(object):
         ).get
         self.kw = kw
         self.arm = pinholeFrame.header[kw("SEQ_ARM")]
+        self.dateObs = pinholeFrame.header[kw("DATE_OBS")]
 
         # DETECTOR PARAMETERS LOOKUP OBJECT
         self.detectorParams = detector_lookup(
@@ -263,6 +270,8 @@ class create_dispersion_map(object):
         ylow = int(np.max([y - windowHalf, 0]))
         yup = int(np.min([y + windowHalf, pinholeFrame.shape[0]]))
         stamp = pinholeFrame[ylow:yup, xlow:xup]
+        # CONVERT TO MASKED ARRAY
+        stamp = np.asanyarray(stamp)
 
         # USE DAOStarFinder TO FIND LINES WITH 2D GUASSIAN FITTING
         mean, median, std = sigma_clipped_stats(stamp, sigma=3.0)
@@ -518,7 +527,7 @@ class create_dispersion_map(object):
             fig = plt.figure(figsize=(6, 11), constrained_layout=True)
         gs = fig.add_gridspec(6, 4)
 
-        # CREATE THE GID OF AXES
+        # CREATE THE GRID OF AXES
         toprow = fig.add_subplot(gs[0:2, :])
         midrow = fig.add_subplot(gs[2:4, :])
         bottomleft = fig.add_subplot(gs[4:, 0:2])
@@ -530,8 +539,9 @@ class create_dispersion_map(object):
         toprow.set_title(
             "observed arc-line positions (post-clipping)", fontsize=10)
 
-        x = np.ones(orderPixelTable.shape[0]) * \
-            self.pinholeFrame.data.shape[1] - orderPixelTable["observed_x"]
+        x = orderPixelTable["observed_x"]
+        # x = np.ones(orderPixelTable.shape[0]) * \
+        #     self.pinholeFrame.data.shape[1] - orderPixelTable["observed_x"]
         toprow.scatter(orderPixelTable["observed_y"],
                        x, marker='x', c='red', s=4)
 
@@ -546,8 +556,9 @@ class create_dispersion_map(object):
         midrow.set_title(
             "global dispersion solution", fontsize=10)
 
-        xfit = np.ones(orderPixelTable.shape[0]) * \
-            self.pinholeFrame.data.shape[1] - orderPixelTable["fit_x"]
+        xfit = orderPixelTable["fit_x"]
+        # xfit = np.ones(orderPixelTable.shape[0]) * \
+        #     self.pinholeFrame.data.shape[1] - orderPixelTable["fit_x"]
         midrow.scatter(orderPixelTable["fit_y"],
                        xfit, marker='x', c='blue', s=4)
 
