@@ -31,6 +31,7 @@ import unicodecsv as csv
 from fundamentals.renderer import list_of_dictionaries
 from soxspipe.commonutils.filenamer import filenamer
 import pandas as pd
+from datetime import datetime, date, time
 
 
 class detect_order_edges(_base_detect):
@@ -43,6 +44,9 @@ class detect_order_edges(_base_detect):
         - ``flatFrame`` -- the flat frame to detect the order edges on
         - ``orderCentreTable`` -- the order centre table
         - ``recipeName`` -- name of the recipe as it appears in the settings dictionary
+        - ``verbose`` -- verbose. True or False. Default *False*
+        - ``qcTable`` -- the data frame to collect measured QC metrics 
+        - ``productsTable`` -- the data frame to collect output products
 
     **Usage:**
 
@@ -59,9 +63,12 @@ class detect_order_edges(_base_detect):
         flatFrame=flatFrame,
         orderCentreTable=orderCentreTable,
         settings=settings,
-        recipeName="soxs-mflat"
+        recipeName="soxs-mflat",
+        verbose=False,
+        qcTable=False,
+        productsTable=False
     )
-    edges.get()
+    productsTable, qcTable = edges.get()
     ```
     """
 
@@ -71,11 +78,15 @@ class detect_order_edges(_base_detect):
             flatFrame,
             orderCentreTable,
             settings=False,
-            recipeName="soxs-mflat"
+            recipeName="soxs-mflat",
+            verbose=False,
+            qcTable=False,
+            productsTable=False
     ):
         self.log = log
         log.debug("instansiating a new 'detect_order_edges' object")
         self.settings = settings
+        self.recipeName = recipeName
         if recipeName:
             self.recipeSettings = settings[recipeName]
         else:
@@ -83,7 +94,9 @@ class detect_order_edges(_base_detect):
 
         self.orderCentreTable = orderCentreTable
         self.flatFrame = flatFrame
-        # xt-self-arg-tmpx
+        self.verbose = verbose
+        self.qc = qcTable
+        self.products = productsTable
 
         # KEYWORD LOOKUP OBJECT - LOOKUP KEYWORD FROM DICTIONARY IN RESOURCES
         # FOLDER
@@ -93,6 +106,7 @@ class detect_order_edges(_base_detect):
         ).get
         kw = self.kw
         self.arm = flatFrame.header[kw("SEQ_ARM")]
+        self.dateObs = flatFrame.header[kw("DATE_OBS")]
 
         # DETECTOR PARAMETERS LOOKUP OBJECT
         self.detectorParams = detector_lookup(
@@ -114,6 +128,8 @@ class detect_order_edges(_base_detect):
         """
         self.log.debug('starting the ``get`` method')
 
+        print("\n# DETECTING THE ORDER EDGES FROM MASTER-FLAT FRAME")
+
         orderTablePath = None
 
         # GET PARAMETERS FROM SETTINGS
@@ -133,6 +149,7 @@ class detect_order_edges(_base_detect):
             orderPolyTable["ymin"].values, orderPolyTable["ymax"].values))
 
         # ADD MIN AND MAX FLUX THRESHOLDS TO ORDER TABLE
+        print("\tDETERMINING ORDER FLUX THRESHOLDS")
         orderPolyTable["maxThreshold"] = np.nan
         orderPolyTable["minThreshold"] = np.nan
         orderPolyTable = orderPolyTable.apply(
@@ -146,6 +163,7 @@ class detect_order_edges(_base_detect):
             orderPixelTable.loc[(orderPixelTable["order"] == o), ["minThreshold", "maxThreshold"]] = orderPolyTable.loc[
                 (orderPolyTable["order"] == o), ["minThreshold", "maxThreshold"]].values
 
+        print("\tMEASURING PIXEL-POSITIONS AT ORDER-EDGES WHERE FLUX THRESHOLDS ARE MET")
         orderPixelTable["xcoord_upper"] = np.nan
         orderPixelTable["xcoord_lower"] = np.nan
         orderPixelTable = orderPixelTable.apply(
@@ -164,6 +182,7 @@ class detect_order_edges(_base_detect):
         orderMaxLocations = {}
         orderMinLocations = {}
 
+        print("\tFITTING POLYNOMIALS TO MEASURED PIXEL-POSITIONS AT UPPER ORDER-EDGES\n")
         for o in uniqueOrders:
             # ITERATIVELY FIT THE POLYNOMIAL SOLUTIONS TO THE DATA
             coeff, orderPixelTable = self.fit_polynomial(
@@ -174,12 +193,14 @@ class detect_order_edges(_base_detect):
             )
             if not isinstance(coeff, type(None)):
                 orderMaxLocations[o] = coeff
+        sys.stdout.write("\x1b[1A\x1b[2K")
 
         # RENAME SOME INDIVIDUALLY
         orderPixelTable.rename(columns={
             "x_fit": "xcoord_upper_fit", "x_fit_res": "xcoord_upper_fit_res"}, inplace=True)
 
         # REDEFINE UNIQUE ORDERS IN CASE ONE OR MORE IS COMPLETELY MISSING
+        print("\tFITTING POLYNOMIALS TO MEASURED PIXEL-POSITIONS AT LOWER ORDER-EDGES\n")
         uniqueOrders = orderPixelTable['order'].unique()
         for o in uniqueOrders:
             # ITERATIVELY FIT THE POLYNOMIAL SOLUTIONS TO THE DATA
@@ -191,12 +212,13 @@ class detect_order_edges(_base_detect):
             )
             if not isinstance(coeff, type(None)):
                 orderMinLocations[o] = coeff
+        sys.stdout.write("\x1b[1A\x1b[2K")
 
         # RENAME SOME INDIVIDUALLY
         orderPixelTable.rename(columns={
             "x_fit": "xcoord_lower_fit", "x_fit_res": "xcoord_lower_fit_res"}, inplace=True)
 
-        # SORT CENTRE TRACE COEFFICIENT OUTPUT TO PANDAS DATAFRAME
+        # SORT UPPER EDGE TRACE COEFFICIENT OUTPUT TO PANDAS DATAFRAME
         columnsNames = ["order", "degy_edgeup"]
         coeffColumns = [f'edgeup_c{i}' for i in range(0, self.polyDeg + 1)]
         columnsNames.extend(coeffColumns)
@@ -210,7 +232,7 @@ class detect_order_edges(_base_detect):
                 n_coeff += 1
         edgeUpTable = pd.DataFrame(myDict)
 
-        # SORT CENTRE TRACE COEFFICIENT OUTPUT TO PANDAS DATAFRAME
+        # SORT LOWER EDGE TRACE COEFFICIENT OUTPUT TO PANDAS DATAFRAME
         columnsNames = ["order", "degy_edgelow"]
         coeffColumns = [f'edgelow_c{i}' for i in range(0, self.polyDeg + 1)]
         columnsNames.extend(coeffColumns)
@@ -231,6 +253,7 @@ class detect_order_edges(_base_detect):
             edgeLowTable, on=["order"], how='outer')
 
         # GENERATE AN OUTPUT PLOT OF RESULTS AND FITTING RESIDUALS
+        print("\tMEASURING AND PLOTTING RESIDUALS OF FITS")
         allResiduals = np.concatenate((orderPixelTable[
             'xcoord_lower_fit_res'], orderPixelTable['xcoord_upper_fit_res']))
         plotPath = self.plot_results(
@@ -252,12 +275,56 @@ class detect_order_edges(_base_detect):
         mean_res = np.mean(np.abs(allResiduals))
         std_res = np.std(np.abs(allResiduals))
 
-        print(f'\nThe order edge polynomials fitted observed order edge positions with a mean residual of {mean_res:2.2f} pixels (stdev = {std_res:2.2f} pixels)')
+        orderTablePath = os.path.abspath(orderTablePath)
+        plotPath = os.path.abspath(plotPath)
+        orderTableName = os.path.basename(orderTablePath)
+        plotName = os.path.basename(plotPath)
 
-        print(f'\nFind results of the order edge fitting here: {plotPath}')
+        utcnow = datetime.utcnow()
+        utcnow = utcnow.strftime("%Y-%m-%dT%H:%M:%S")
+
+        # RECORD PRODUCTS AND QCs
+        if not isinstance(self.products, bool):
+            self.products = self.products.append({
+                "soxspipe_recipe": self.recipeName,
+                "product_label": "ORDER_LOC",
+                "product_desc": "table of coefficients from polynomial fits to order locations",
+                "file_name": orderTableName,
+                "file_type": "CSV",
+                "obs_date_utc": self.dateObs,
+                "reduction_date_utc": utcnow,
+                "file_path": orderTablePath
+            }, ignore_index=True)
+            self.products = self.products.append({
+                "soxspipe_recipe": self.recipeName,
+                "product_label": "",
+                "product_desc": "visualisation of goodness of order edge fitting",
+                "file_name": plotName,
+                "file_type": "PDF",
+                "obs_date_utc": self.dateObs,
+                "reduction_date_utc": utcnow,
+                "file_path": plotPath
+            }, ignore_index=True)
+        if not isinstance(self.qc, bool):
+            self.qc = self.qc.append({
+                "soxspipe_recipe": self.recipeName,
+                "qc_name": "order edge fit mean residual",
+                "qc_value": mean_res,
+                "qc_unit": "pixels",
+                "obs_date_utc": self.dateObs,
+                "reduction_date_utc": utcnow
+            }, ignore_index=True)
+            self.qc = self.qc.append({
+                "soxspipe_recipe": self.recipeName,
+                "qc_name": "order edge fit residual stdev",
+                "qc_value": std_res,
+                "qc_unit": "pixels",
+                "obs_date_utc": self.dateObs,
+                "reduction_date_utc": utcnow
+            }, ignore_index=True)
 
         self.log.debug('completed the ``get`` method')
-        return orderTablePath
+        return self.products, self.qc
 
     def plot_results(
             self,
@@ -496,11 +563,11 @@ class detect_order_edges(_base_detect):
         # REPORT THE EXACT PIXEL POSTION AT THE FLUX THRESHOLD
         xmax = xmaxguess - \
             (threshold - medSlide[xmaxguess]) / \
-            (medSlide[xmaxguess - 1] - medSlide[xmaxguess])
+            (medSlide[xmaxguess - 1] - medSlide[xmaxguess]) - 2
 
         xmin = xminguess + \
             (threshold - medSlide[xminguess]) / \
-            (medSlide[xminguess + 1] - medSlide[xminguess])
+            (medSlide[xminguess + 1] - medSlide[xminguess]) + 2
 
         # IF THE WIDTH BETWEEN MIN AND MAX IS TOO SMALL THEN REJECT
         if xmax - xmin < 10:
@@ -510,7 +577,7 @@ class detect_order_edges(_base_detect):
             orderData["xcoord_lower"] = xmin + int(x - halfSlice)
 
         # SANITY CHECK PLOT OF CROSS-SECTION
-        if 1 == 0 and random.randint(1, 101) < 2:
+        if 1 == 0 and random.randint(1, 5001) < 2:
             # CHECK THE SLICE POINTS IF NEEDED
             x = np.arange(0, len(slice))
             plt.figure(figsize=(8, 5))
