@@ -20,6 +20,7 @@ from astropy.nddata import CCDData
 from astropy import units as u
 from astropy.stats import mad_std
 import ccdproc
+import pandas as pd
 from astropy.nddata.nduncertainty import StdDevUncertainty
 from ccdproc import Combiner
 from soxspipe.commonutils import set_of_files
@@ -28,6 +29,7 @@ from soxspipe.commonutils import detector_lookup
 from datetime import datetime
 from soxspipe.commonutils import filenamer
 import shutil
+from tabulate import tabulate
 
 
 class _base_recipe_(object):
@@ -37,6 +39,7 @@ class _base_recipe_(object):
     **Key Arguments:**
         - ``log`` -- logger
         - ``settings`` -- the settings dictionary
+        - ``verbose`` -- verbose. True or False. Default *False*
 
     **Usage**
 
@@ -46,7 +49,8 @@ class _base_recipe_(object):
     def __init__(
             self,
             log,
-            settings=False
+            settings=False,
+            verbose=False
     ):
         self.log = log
         log.debug("instansiating a new '__init__' object")
@@ -58,9 +62,30 @@ class _base_recipe_(object):
         self.calibrationRootPath = self._absolute_path(
             settings["calibration-data-root"])
 
+        self.verbose = verbose
         # SET LATER WHEN VERIFYING FRAMES
         self.arm = None
         self.detectorParams = None
+        self.dateObs = None
+
+        # DATAFRAMES TO COLLECT QCs AND PRODUCTS
+        self.qc = pd.DataFrame({
+            "soxspipe_recipe": [],
+            "qc_name": [],
+            "qc_value": [],
+            "qc_unit": [],
+            "obs_date_utc": [],
+            "reduction_date_utc": []
+        })
+        self.products = pd.DataFrame({
+            "soxspipe_recipe": [],
+            "product_label": [],
+            "file_name": [],
+            "file_type": [],
+            "obs_date_utc": [],
+            "reduction_date_utc": [],
+            "file_path": []
+        })
 
         # KEYWORD LOOKUP OBJECT - LOOKUP KEYWORD FROM DICTIONARY IN RESOURCES
         # FOLDER
@@ -259,14 +284,15 @@ class _base_recipe_(object):
         filepaths = self.inputFrames.files_filtered(include_path=True)
 
         frameCount = len(filepaths)
-        print("# PREPARING %(frameCount)s RAW FRAMES - TRIMMING OVERSCAN, CONVERTING TO ELECTRON COUNTS, GENERATING UNCERTAINTY MAPS AND APPENDING DEFAULT BAD-PIXEL MASK" % locals())
+        print("\n# PREPARING %(frameCount)s RAW FRAMES - TRIMMING OVERSCAN, CONVERTING TO ELECTRON COUNTS, GENERATING UNCERTAINTY MAPS AND APPENDING DEFAULT BAD-PIXEL MASK" % locals())
         preframes = []
         preframes[:] = [self._prepare_single_frame(
             frame=frame, save=save) for frame in filepaths]
         sof = set_of_files(
             log=self.log,
             settings=self.settings,
-            inputFrames=preframes
+            inputFrames=preframes,
+            verbose=self.verbose
         )
         preframes, supplementaryInput = sof.get()
         preframes.sort([kw('MJDOBS').lower()])
@@ -494,6 +520,8 @@ class _base_recipe_(object):
         HDUList.writeto(filepath, output_verify='exception',
                         overwrite=overwrite, checksum=True)
 
+        filepath = os.path.abspath(filepath)
+
         self.log.debug('completed the ``write`` method')
         return filepath
 
@@ -546,13 +574,18 @@ class _base_recipe_(object):
             recipe]["clipping-iteration-count"]
 
         # LIST OF CCDDATA OBJECTS NEEDED BY COMBINER OBJECT
-        # ccds = [c for c in self.inputFrames.ccds()]
         if not isinstance(frames, list):
             ccds = [c for c in frames.ccds(ccd_kwargs={"hdu_uncertainty": 'ERRS',
                                                        "hdu_mask": 'QUAL', "hdu_flags": 'FLAGS', "key_uncertainty_type": 'UTYPE'})]
         else:
             ccds = frames
 
+        imageType = ccds[0].header[kw("DPR_TYPE").lower()].replace(",", "-")
+        imageTech = ccds[0].header[kw("DPR_TECH").lower()].replace(",", "-")
+        imageCat = ccds[0].header[kw("DPR_CATG").lower()].replace(",", "-")
+
+        print(f"\n# MEAN COMBINING {len(ccds)} {arm} {imageCat} {imageTech} {imageType} FRAMES")
+
         # COMBINE MASKS AND THEN RESET
         combinedMask = ccds[0].mask
         for c in ccds:
@@ -564,14 +597,14 @@ class _base_recipe_(object):
         # MASKED IN ALL INDIVIDUAL IMAGES ARE MASK IN THE FINAL COMBINED IMAGE
         combiner = Combiner(ccds)
 
-        print(f"\n# SIGMA-CLIPPING PIXEL WITH OUTLYING VALUES IN INDIVIDUAL {imageType} FRAMES")
+        # print(f"\n# SIGMA-CLIPPING PIXEL WITH OUTLYING VALUES IN INDIVIDUAL {imageType} FRAMES")
         # PRINT SOME INFO FOR USER
         badCount = combinedMask.sum()
         totalPixels = np.size(combinedMask)
         percent = (float(badCount) / float(totalPixels)) * 100.
-        print(f"The basic bad-pixel mask for the {arm} detector {imageType} frames contains {badCount} pixels ({percent:0.2}% of all pixels)")
+        print(f"\tThe basic bad-pixel mask for the {arm} detector {imageType} frames contains {badCount} pixels ({percent:0.2}% of all pixels)")
 
-        # GENERATE A MASK FOR EACH OF THE INDIVIDUAL INOUT FRAMES - USING
+        # GENERATE A MASK FOR EACH OF THE INDIVIDUAL INPUT FRAMES - USING
         # MEDIAN WITH MEDIAN ABSOLUTE DEVIATION (MAD) AS THE DEVIATION FUNCTION
         old_n_masked = -1
         # THIS IS THE SUM OF BAD-PIXELS IN ALL INDIVIDUAL FRAME MASKS
@@ -587,11 +620,12 @@ class _base_recipe_(object):
             extra = ""
             if diff == 0:
                 extra = " - we're done"
-            print("    Clipping iteration %(iteration)s finds %(diff)s more rogue pixels in the set of input frames%(extra)s" % locals())
+            if self.verbose:
+                print("\tClipping iteration %(iteration)s finds %(diff)s more rogue pixels in the set of input frames%(extra)s" % locals())
             iteration += 1
 
         # GENERATE THE COMBINED MEDIAN
-        print("\n# MEAN COMBINING FRAMES - WITH UPDATED BAD-PIXEL MASKS")
+        # print("\n# MEAN COMBINING FRAMES - WITH UPDATED BAD-PIXEL MASKS")
         combined_frame = combiner.average_combine()
 
         # RECOMBINE THE COMBINED MASK FROM ABOVE
@@ -610,7 +644,7 @@ class _base_recipe_(object):
         # CALCULATE NEW PIXELS ADDED TO MASK
         newBadCount = combined_frame.mask.sum()
         diff = newBadCount - badCount
-        print("%(diff)s new pixels made it into the combined bad-pixel map" % locals())
+        print("\t%(diff)s new pixels made it into the combined bad-pixel map" % locals())
 
         self.log.debug('completed the ``clip_and_stack`` method')
         return combined_frame
@@ -691,5 +725,46 @@ class _base_recipe_(object):
         self.log.debug('completed the ``subtract_calibrations`` method')
         return calibration_subtracted_frame
 
+    def report_output(
+            self,
+            rformat="stdout"):
+        """*a method to report QC values alongside imtermediate and final products*
+
+        **Key Arguments:**
+            - ``rformat`` -- the format to outout reports as. Default *stdout*. [stdout|....]
+
+        **Return:**
+            - None
+
+        **Usage:**
+
+        ```python
+        usage code 
+        ```
+
+        ---
+
+        ```eval_rst
+        .. todo::
+
+            - add usage info
+            - create a sublime snippet for usage
+            - write a command-line tool for this method
+            - update package tutorial with command-line tool info if needed
+        ```
+        """
+        self.log.debug('starting the ``report_output`` method')
+
+        if not self.verbose:
+            # REMOVE COLUMN FROM DATA FRAME
+            self.products.drop(columns=['file_path'], inplace=True)
+
+        if rformat == "stdout":
+            print(tabulate(self.qc, headers='keys', tablefmt='psql'))
+            print(tabulate(self.products, headers='keys', tablefmt='psql'))
+
+        self.log.debug('completed the ``report_output`` method')
+        return None
+
     # use the tab-trigger below for new method
     # xt-class-method
