@@ -33,6 +33,7 @@ class soxs_mbias(_base_recipe_):
     - ``log`` -- logger
     - ``settings`` -- the settings dictionary
     - ``inputFrames`` -- input fits frames. Can be a directory, a set-of-files (SOF) file or a list of fits frame paths.
+    - ``verbose`` -- verbose. True or False. Default *False*
 
     **Usage**
 
@@ -58,7 +59,8 @@ class soxs_mbias(_base_recipe_):
             self,
             log,
             settings=False,
-            inputFrames=[]
+            inputFrames=[],
+            verbose=False
 
     ):
         # INHERIT INITIALISATION FROM  _base_recipe_
@@ -66,8 +68,10 @@ class soxs_mbias(_base_recipe_):
         self.log = log
         log.debug("instansiating a new 'soxs_mbias' object")
         self.settings = settings
-        self.recipeSettings = settings["soxs-mbias"]
         self.inputFrames = inputFrames
+        self.verbose = verbose
+        self.recipeName = "soxs-mbias"
+        self.recipeSettings = settings[self.recipeName]
         # xt-self-arg-tmpx
 
         # INITIAL ACTIONS
