@@ -32,6 +32,7 @@ class soxs_mdark(_base_recipe_):
         - ``log`` -- logger
         - ``settings`` -- the settings dictionary
         - ``inputFrames`` -- input fits frames. Can be a directory, a set-of-files (SOF) file or a list of fits frame paths.
+        - ``verbose`` -- verbose. True or False. Default *False*
 
     **Usage**
 
@@ -57,7 +58,8 @@ class soxs_mdark(_base_recipe_):
             self,
             log,
             settings=False,
-            inputFrames=[]
+            inputFrames=[],
+            verbose=False
 
     ):
         # INHERIT INITIALISATION FROM  _base_recipe_
@@ -65,8 +67,10 @@ class soxs_mdark(_base_recipe_):
         self.log = log
         log.debug("instansiating a new 'soxs_mdark' object")
         self.settings = settings
-        self.recipeSettings = settings["soxs-mdark"]
         self.inputFrames = inputFrames
+        self.verbose = verbose
+        self.recipeName = "soxs-mdark"
+        self.recipeSettings = settings[self.recipeName]
         # xt-self-arg-tmpx
 
         # INITIAL ACTIONS
@@ -86,10 +90,11 @@ class soxs_mdark(_base_recipe_):
         sys.stdout.write("\x1b[1A\x1b[2K")
         print("# VERIFYING INPUT FRAMES - ALL GOOD")
 
-        print("\n# RAW INPUT FRAMES - SUMMARY")
         # SORT IMAGE COLLECTION
         self.inputFrames.sort(['mjd-obs'])
-        print(self.inputFrames.summary, "\n")
+        if self.verbose:
+            print("# RAW INPUT FRAMES - SUMMARY")
+            print(self.inputFrames.summary, "\n")
 
         # PREPARE THE FRAMES - CONVERT TO ELECTRONS, ADD UNCERTAINTY AND MASK
         # EXTENSIONS
