@@ -30,7 +30,9 @@ import pandas as pd
 from soxspipe.commonutils import subtract_background
 from os.path import expanduser
 from soxspipe.commonutils.filenamer import filenamer
-from astropy.stats import sigma_clip, mad_std
+from astropy.stats import sigma_clip, mad_std, sigma_clipped_stats
+from datetime import datetime
+from soxspipe.commonutils.toolkit import generic_quality_checks, spectroscopic_image_quality_checks
 
 
 class soxs_mflat(_base_recipe_):
@@ -42,6 +44,7 @@ class soxs_mflat(_base_recipe_):
         - ``log`` -- logger
         - ``settings`` -- the settings dictionary
         - ``inputFrames`` -- input fits frames. Can be a directory, a set-of-files (SOF) file or a list of fits frame paths.
+        - ``verbose`` -- verbose. True or False. Default *False*  
 
     **Usage**
 
@@ -68,7 +71,8 @@ class soxs_mflat(_base_recipe_):
             self,
             log,
             settings=False,
-            inputFrames=[]
+            inputFrames=[],
+            verbose=False
 
     ):
         # INHERIT INITIALISATION FROM  _base_recipe_
@@ -77,8 +81,11 @@ class soxs_mflat(_base_recipe_):
         self.log = log
         log.debug("instansiating a new 'soxs_mflat' object")
         self.settings = settings
-        self.recipeSettings = settings["soxs-mflat"]
         self.inputFrames = inputFrames
+        self.verbose = verbose
+        self.recipeName = "soxs-mflat"
+        self.recipeSettings = settings[self.recipeName]
+
         # xt-self-arg-tmpx
 
         # CONVERT INPUT FILES TO A CCDPROC IMAGE COLLECTION (inputFrames >
@@ -97,10 +104,11 @@ class soxs_mflat(_base_recipe_):
         sys.stdout.write("\x1b[1A\x1b[2K")
         print("# VERIFYING INPUT FRAMES - ALL GOOD")
 
-        print("\n# RAW INPUT FRAMES - SUMMARY")
         # SORT IMAGE COLLECTION
         self.inputFrames.sort(['mjd-obs'])
-        print(self.inputFrames.summary, "\n")
+        if self.verbose:
+            print("# RAW INPUT FRAMES - SUMMARY")
+            print(self.inputFrames.summary, "\n")
 
         # PREPARE THE FRAMES - CONVERT TO ELECTRONS, ADD UNCERTAINTY AND MASK
         # EXTENSIONS
@@ -198,6 +206,7 @@ class soxs_mflat(_base_recipe_):
 
         # DIVIDE THROUGH BY FIRST-PASS MASTER FRAME TO REMOVE CROSS-PLANE
         # ILLUMINATION VARIATIONS
+        print("\n# DIVIDING EACH ORIGINAL FLAT FRAME BY FIRST PASS MASTER FLAT")
         exposureFrames = []
         exposureFrames[:] = [
             n.divide(combined_normalised_flat) for n in calibratedFlats]
@@ -218,7 +227,7 @@ class soxs_mflat(_base_recipe_):
             frames=normalisedFlats, recipe="soxs_mflat")
 
         quicklook_image(
-            log=self.log, CCDObject=combined_normalised_flat, show=True)
+            log=self.log, CCDObject=combined_normalised_flat, show=False)
 
         # DETECT THE ORDER EDGES AND UPDATE THE ORDER LOCATIONS TABLE
         edges = detect_order_edges(
@@ -226,31 +235,57 @@ class soxs_mflat(_base_recipe_):
             flatFrame=combined_normalised_flat,
             orderCentreTable=self.supplementaryInput[arm]["ORDER_LOCATIONS"],
             settings=self.settings,
+            qcTable=self.qc,
+            productsTable=self.products
         )
-        orderTablePath = edges.get()
+        self.products, self.qc = edges.get()
+        # FILTER DATA FRAME
+        # FIRST CREATE THE MASK
+        mask = (self.products['product_label'] == "ORDER_LOC")
+        orderTablePath = self.products.loc[mask]["file_path"].values[0]
+
+        self.dateObs = combined_normalised_flat.header[self.kw("DATE_OBS")]
 
         quicklook_image(
             log=self.log, CCDObject=combined_normalised_flat, show=False)
 
-        background = subtract_background(
-            log=self.log,
-            frame=combined_normalised_flat,
-            orderTable=orderTablePath,
-            settings=self.settings
-        )
-        backgroundFrame, mflat = background.subtract()
+        mflat = self.mask_low_sens_and_inter_order_to_unity(
+            frame=combined_normalised_flat, orderTablePath=orderTablePath)
+
+        # background = subtract_background(
+        #     log=self.log,
+        #     frame=combined_normalised_flat,
+        #     orderTable=orderTablePath,
+        #     settings=self.settings
+        # )
+        # backgroundFrame, mflat = background.subtract()
 
         quicklook_image(
-            log=self.log, CCDObject=mflat, show=True)
+            log=self.log, CCDObject=mflat, show=False)
 
         home = expanduser("~")
         outDir = self.settings["intermediate-data-root"].replace("~", home)
         # filePath = f"{outDir}/first_iteration_{arm}_master_flat.fits"
 
+        # WRITE MFLAT TO FILE
+
         productPath = self._write(
             mflat, outDir, overwrite=True)
+        utcnow = datetime.utcnow()
+        utcnow = utcnow.strftime("%Y-%m-%dT%H:%M:%S")
+        basename = os.path.basename(productPath)
+        self.products = self.products.append({
+            "soxspipe_recipe": self.recipeName,
+            "product_label": "MFLAT",
+            "file_name": basename,
+            "file_type": "FITS",
+            "obs_date_utc": self.dateObs,
+            "reduction_date_utc": utcnow,
+            "product_desc": f"{self.arm} master spectroscopic flat frame",
+            "file_path": productPath
+        }, ignore_index=True)
 
-        if 1 == 1:
+        if 1 == 0:
             filename = filenamer(
                 log=self.log,
                 frame=mflat,
@@ -259,9 +294,26 @@ class soxs_mflat(_base_recipe_):
             filename = filename.replace(".fits", "_background.fits")
             filepath = self._write(
                 backgroundFrame, outDir, filename=filename, overwrite=True)
-            print(f"\nbackground frame saved to {filepath}\n")
+            filepath = os.path.abspath(filepath)
+            self.products = self.products.append({
+                "soxspipe_recipe": self.recipeName,
+                "product_label": "",
+                "file_name": filename,
+                "file_type": "FITS",
+                "obs_date_utc": self.dateObs,
+                "reduction_date_utc": utcnow,
+                "product_desc": f"modelled scatter background light image (removed from master flat)",
+                "file_path": backgroundFrame
+            }, ignore_index=True)
+
+        # ADD QUALITY CHECKS
+        self.qc = generic_quality_checks(
+            log=self.log, frame=mflat, settings=self.settings, recipeName=self.recipeName, qcTable=self.qc)
+        self.qc = spectroscopic_image_quality_checks(
+            log=self.log, frame=mflat, settings=self.settings, recipeName=self.recipeName, qcTable=self.qc, orderTablePath=orderTablePath)
 
         self.clean_up()
+        self.report_output()
 
         self.log.debug('completed the ``produce_product`` method')
         return productPath
@@ -321,6 +373,7 @@ class soxs_mflat(_base_recipe_):
         calibratedFlats = []
         if not darkCollection and bias:
             for flat in flats:
+                print("\n# SUBTRACTING MASTER BIAS FROM FRAMES")
                 calibratedFlats.append(self.subtract_calibrations(
                     inputFrame=flat, master_bias=bias, dark=None))
 
@@ -331,6 +384,7 @@ class soxs_mflat(_base_recipe_):
                         for h in darkCollection.headers()]
             darks = [c for c in darkCollection.ccds(ccd_kwargs={
                 "hdu_uncertainty": 'ERRS', "hdu_mask": 'QUAL', "hdu_flags": 'FLAGS', "key_uncertainty_type": 'UTYPE'})]
+            print("\n# SUBTRACTING MASTER DARK/OFF-LAMP FROM FRAMES")
             for flat in flats:
                 mjd = flat.header[kw("MJDOBS").lower()]
                 matchValue, matchIndex = nearest_neighbour(
@@ -404,6 +458,7 @@ class soxs_mflat(_base_recipe_):
             quicklook_image(log=self.log, CCDObject=maskedFrame,
                             show=False, ext=None)
 
+        print("\n# NORMALISING FLAT FRAMES TO THEIR MEAN EXPOSURE LEVEL")
         for frame, exp in zip(inputFlats, exposureFrames):
             maskedFrame = ma.array(exp.data, mask=mask)
 
@@ -423,6 +478,89 @@ class soxs_mflat(_base_recipe_):
         self.log.debug('completed the ``normalise_flats`` method')
         return normalisedFrames
 
+    def mask_low_sens_and_inter_order_to_unity(
+            self,
+            frame,
+            orderTablePath):
+        """*set inter-order pixels to unity and and low-sensitivity pixels to bad-pixel mask*
+
+        **Key Arguments:**
+            - ``frame`` -- the frame to work on
+            - ``orderTablePath`` -- path to the order table
+
+        **Return:**
+            - ``frame`` -- with inter-order pixels set to 1 and BPM updated with low-sensitivity pixels
+        """
+        self.log.debug(
+            'starting the ``mask_low_sens_and_inter_order_to_unity`` method')
+
+        print("\n# CLIPPING LOW-SENSITIVITY PIXELS AND SETTING INTER-ORDER AREA TO UNITY")
+
+        # UNPACK THE ORDER TABLE
+        orderTableMeta, orderTablePixels = unpack_order_table(
+            log=self.log, orderTablePath=orderTablePath)
+
+        # BAD PIXEL COUNT AT START
+        originalBPM = np.copy(frame.mask)
+
+        interOrderMask = np.ones_like(frame.data)
+        xcoords_up = orderTablePixels["xcoord_edgeup"].values
+        xcoords_low = orderTablePixels["xcoord_edgelow"].values
+        ycoords = orderTablePixels["ycoord"].values
+        xcoords_up = xcoords_up.astype(int)
+        xcoords_low = xcoords_low.astype(int)
+
+        # UPDATE THE MASK TO INCLUDE INTER-ORDER BACKGROUND
+        for u, l, y in zip(xcoords_up, xcoords_low, ycoords):
+            interOrderMask[y][l:u] = 0
+
+        # CONVERT TO BOOLEAN MASK AND MERGE WITH BPM
+        interOrderMask = ma.make_mask(interOrderMask)
+        frame.mask = (interOrderMask == 1) | (frame.mask == 1)
+
+        # PLOT MASKED FRAMES TO CHECK
+        quicklook_image(log=self.log, CCDObject=frame,
+                        show=False, ext=None)
+
+        beforeMask = np.copy(frame.mask)
+
+        # SIGMA-CLIP THE LOW-SENSITIVITY PIXELS
+        frameClipped = sigma_clip(
+            frame, sigma_lower=self.settings["soxs-mflat"]["low-sensitivity-clipping-simga"], sigma_upper=2000, maxiters=5, cenfunc='median', stdfunc=mad_std)
+
+        # PLOT MASKED FRAMES TO CHECK
+        quicklook_image(log=self.log, CCDObject=frame,
+                        show=False, ext=None)
+
+        lowSensitivityPixelMask = (frameClipped.mask == 1) & (beforeMask != 1)
+        lowSensPixelCount = lowSensitivityPixelMask.sum()
+
+        utcnow = datetime.utcnow()
+        utcnow = utcnow.strftime("%Y-%m-%dT%H:%M:%S")
+
+        self.qc = self.qc.append({
+            "soxspipe_recipe": self.recipeName,
+            "qc_name": "low sensitivity pixel count",
+            "qc_value": lowSensPixelCount,
+            "qc_unit": "pixels",
+            "obs_date_utc": self.dateObs,
+            "reduction_date_utc": utcnow
+        }, ignore_index=True)
+        print(f"        {lowSensPixelCount} low-sensitivity pixels added to bad-pixel mask")
+
+        frame.mask = (lowSensitivityPixelMask == 1) | (originalBPM == 1)
+
+        # SET INTRA-ORDER TO 1
+        frame.data[interOrderMask] = 1
+
+        # PLOT MASKED FRAMES TO CHECK
+        quicklook_image(log=self.log, CCDObject=frame,
+                        show=False, ext=None)
+
+        self.log.debug(
+            'completed the ``mask_low_sens_and_inter_order_to_unity`` method')
+        return frame
+
     # use the tab-trigger below for new method
     # xt-class-method
 
